# encoding: UTF-8
class Transfer < ActiveRecord::Base
  include Uuidextension
  has_paper_trail
  self.table_name = 'transfers'
  before_create :nastav_date_inserted
  before_save :create_default_epoch_if_does_not_exist
  after_save :update_splits

# versioned :exclude => :amount # see http://asciicasts.com/episodes/177-model-versioning
# http://ucet.kub.cz/transfers/69e79baa-7651-11e0-80f7-7071bcccc8a2 - vyplneni 541 do c. dokladu

  belongs_to :account_from, :class_name=>"Account", :foreign_key=> "id_account_from"
  belongs_to :account_to,   :class_name=>"Account", :foreign_key=> "id_account_to"
  belongs_to :owner,   :class_name=>"Owner", :foreign_key=> "id_account_owner"
  has_many :documents, :through => :attachements
  has_many :attachements, :dependent => :destroy
  belongs_to :next_transfer_in_chain, :class_name=>"Transfer", :foreign_key=> "id_transfer_reimbursed"
  has_one :previous_transfer_in_chain, :class_name=>"Transfer", :foreign_key=> "id_transfer_reimbursed"
  belongs_to :epoch
  has_many :splits, :autosave => true, :dependent => :destroy
  belongs_to :partner, :foreign_key=>:vendor, :primary_key=>:nazev
  has_and_belongs_to_many :tags

  validates_presence_of :account_from
  validates_presence_of :account_to
  validates_presence_of :owner
  validates_presence_of :datum
  validates_presence_of :popis
  validates_presence_of :amount

  serialize :amount
  attr_writer :reversed # vlastnost kterou ma jen transfer v ramci uctu

  scope :selected, :conditions => {:selected => 1}, :order => 'datum2,seq,id'
  scope :after, lambda { |ago|  where("transfers.datum >= ?", ago) }  

  default_scope do
    if $user == 'petra' or $user=='tonda'
      after('2010-01-01').where(:id_account_owner => OWNER_PALUBA_ID)
    else
      scoped
    end
  end

  def self.find_subtransfers_from ( account_id )
    find_by_sql([ 'SELECT * FROM rich_transfers WHERE id_account_pow = ?', account_id ])
  end
  def self.find_subtransfers ( account_id )
    find_by_sql([ 'SELECT * FROM rich_transfers WHERE id_account_pow = ?', account_id ])
  end
  def self.new_from_template(template, parametres)
    if template.nil? then return nil end
    xml = String.new(template.xml)
    @contains_functions = xml.match(/\[\{/)
    
    if parametres
      parametres.each do |key,value|
        xml = xml.gsub(/\[#{key}\]/, value).gsub(/\\\[/,'[').gsub(/\\\]/,']')
        if @contains_functions  # here you can define various function through which the value is recalculated
          ## usage [{Account.find_by_popis}key] is something like #{Account.find_by_popis([key]).id}
          if xml.match(/\[\{Account.find_by_popis\}/) and !Account.find_by_popis(value).nil? 
            xml = xml.gsub(/\[\{Account.find_by_popis\}#{key}\]/, Account.find_by_popis(value).id).gsub(/\\\[/,'[').gsub(/\\\]/,']')
          end
           
          ## usage [{minus}key] ... inverts sign of result, i.e. something like -[key]
          if xml.match(/\[\{minus\}/)  
            if value.match(/^\s*-/) then value.gsub!(/^\s*-/,'') else value = '-'+value end
            xml = xml.gsub(/\[\{minus\}#{key}\]/, value ).gsub(/\\\[/,'[').gsub(/\\\]/,']')
          end
        end
      end
    end
    temp_parametres = Hash.from_xml(xml)
    if temp_parametres['transfer'].nil? || temp_parametres['transfer']['datum'].nil?
      temp_parametres['transfer']['datum'] = Date.today
    end

    final_parametres=Hash.new()
    temp_parametres['transfer'].each do |key,value|
      final_parametres[key.to_sym]= value
    end
    Transfer.new(final_parametres)
  end
  def filenames
    @result = ''
    documents.each{|x| @result = @result + x.filename.to_s }
    return @result
  end
  def documents_filename_comma_list
    documents.collect{|doc| doc.filename.to_s }.join(', ')
  end
  def htmlfilenames
    documents.collect {|x| 
      "<a style=\"white-space:nowrap\" href=\"/documents/#{x.filename}\">#{x.filename}</a> " 
    }.join.html_safe
  end
  def my_seq
    if my_side_defined? and seq.andand[/^(.*?)\/(.*)/]
      reversed? ? $2 : $1
    else
      seq
    end
  end
  def my_slash_other_seq
    if my_side_defined? and seq[/^(.*?)\/(.*)/] and reversed?
      $2+"/"+$1
    else
      seq
    end
  end
  def amount_CZK_spent # vypocita kolik se v CZK skutecne uz utratilo (vynecha planovane platby)
    if account_from.id == '9a9726a4-f5b6-11dc-85fa-0013d4bc1a7d'  # US account
      return Transfer.USD2CZK(amount,datum)
    elsif account_from.id == '768409d0-ffc5-11dc-a064-0013d4bc1a7d'  # Payment calendar
      return 0
    else
      return amount
    end
  end
  def amountQuasiUSD
    pom = self.amount['USD']
    if (pom.nil?) 
      ra = self.read_attribute(:amount)
      if ra.to_s =~ /^([0-9\-+,\.]+)$/  # pokud je to jen cislo tak to budeme povazovat za USD
        pom = ra
      end
    end
    pom
  end
  def self.USD2CZK(usd_amount,datum)
    rates = {
      '2007-01' => 20.91000, '2007-02' => 21.76140, '2007-03' => 21.45370, '2007-04' => 20.9960,
      '2007-05' => 20.68140, '2007-06' => 21.0440,  '2007-07' => 21.27250, '2007-08' => 20.5660,
      '2007-09' => 20.3490,  '2007-10' => 19.38080, '2007-11' => 18.660,   '2007-12' => 17.9870,
      '2008-01' => 18.13370, '2008-02' => 17.46800, '2008-03' => 16.54804, '2008-04' => 16.09080,
      '2008-05' => 16.09319, '2008-06' => 16.03571, '2008-07' => 15.09000, '2008-08' => 15.36192,
      '2008-09' => 16.92698, '2008-10' => 17.32697, '2008-11' => 18.99097, '2008-12' => 20.02396,
      '2009-01' => 19.13214, '2009-02' => 19.25981, '2010-01' => 18.0329,  '2011-01' => 18.3686,
      '2012-01' => 19.3093,  '2013-01' => 18.6061,  '2013-02' => 18.5101,  '2013-03' => 19.1405,
      '2013-04' => 19.6286,  '2013-06' => 19.3281,  '2013-09' => 19.1010,  '2014-01' => 19.4554      
    }
    rate = rates[datum.strftime('%Y-%m')]
    if rate == nil 
      rate = datum.strftime('%Y-%m')<'2007-01' ? rates['2007-01'] : rates['2008-07']
    end
    return rate*usd_amount
  end
  def to_s
    popis.to_s
  end  

### odsud jsou multicash veci  

  def from_splits
    splits.joins{transfer}.where{account_id==transfer.id_account_from}
  end  
  def to_splits
    splits.joins{transfer}.where{account_id==transfer.id_account_to}
  end
  def update_splits(am=nil)
    unless new_record?
      am ||= amount # pokud je amount nil tak si ho musim precist
      
      self.splits.clear
      
      from_split = self.splits.create(:account_id=>self.id_account_from)
      to_split = self.splits.create(:account_id=>self.id_account_to)
      
      from_split.amount = -am
      to_split.amount = am
      
#       self.splits=[from_split,to_split]      
    end
  end
  def amount=(value)
    ## parsuje value a ulozi Hash nebo jen cislo
    h = {}
    case value
     when String
       h = value.to_multicash_h
     when Float, Integer, Bignum, BigDecimal
       h = { $default_commodity => value }
     when Hash
       h = value
     when NilClass
       h = nil
    end
    self.update_splits(h)

    # if only default currency than save only number
    if h.nil? or h.length==0
      write_attribute(:amount,nil)
    elsif (h.length == 1) & (h.keys[0]==$default_commodity)
      write_attribute(:amount, h[$default_commodity]) 
    else 
      write_attribute(:amount, h)
    end
  end
  def amountUSD=(value)
    case value
     when Float, Integer, Bignum, BigDecimal
       h = { 'USD' => value }
     when String,Hash
       h = value
    end
    amount = h
  end
  def amount(options={})  # vzdy vraci hash  
    a = read_attribute(:amount)
    case a
      when Float, Fixnum, String, BigDecimal
        h = { $default_commodity => a }
      when Array
        h = {}
        a.each{ |item| h[item[1]] = item[0] }
      when Hash
        h = a
      when FalseClass
        h = {}
      when NilClass
        if options[:no_zero_default]
          h = nil
        else
          h = {}
        end
    end
    h
  end
  def only_default_currecy?
    h = amount
    (h.length == 0) || ((h.length == 1) && (h.keys[0]==$default_commodity))
  end
  def id_my_account
    if reversed? then id_account_to   else id_account_from end
  end
  def my_account
    if reversed? then account_to      else account_from    end
  end
  def othe1r_account
    Account.find(id_other_account)
  end
  def id_other_account
    if reversed? then id_account_from else id_account_to   end
  end
  def my_other
    if reversed? then account_from    else account_to      end
  end
  def my_amount
    if reversed? then -amount         else amount          end
  end
  def my_balance
    if reversed? then  amount         else -amount         end
  end  
  def reversed?           # according to my_account the transfer goes from to to from
    if @reversed.nil?
      if reversed == 1
        @reversed = true
      elsif reversed == 0
        @reversed = false
      else
        raise "Podivuhodne: my_account not set (reversed=#{reversed.inspect})"
      end
      @reversed
    else
      @reversed
    end
  end
  def my_side_defined?
    if !@reversed.nil?
      true
    else
      if respond_to?(:reversed)
        ((reversed.to_s == '1')or(reversed.to_s == '0'))
      else
        false
      end
    end
  end
  def cislovtextu
    /(AN|CU|ČÚ):?\s*((\d{0,6}-)?\d{2,10}\/\d{4})/ =~ self.popis ? $2 : nil
  end
  def cislouctu           ### TODO: presunout do modelu Partner
    if cislovtextu.nil?
      partner = Partner.find_by_nazev(self.vendor)
      if partner.nil?
        return nil
      else
        return partner.id
      end
    else
      cislovtextu
    end
  end
  def cislouctu_prefix
    if (! cislouctu.nil?) and (! cislouctu.index('-').nil? )
      cislouctu.split('-')[0]
    else
      nil
    end
  end
  def cislouctu_banka
    if (! cislouctu.nil?)
      cislouctu.split('/')[1]
    else
      nil
    end
  end
  def cislouctu_zaklad
    if (! cislouctu.nil?)
      pom = cislouctu.split('/')[0]
      if (! pom.index('-').nil? )
        pom = pom.split('-')[1]
      end
      return pom
    else
      return nil
    end
  end
  def ks
    /KS:?\s*(\d*)/ =~ self.popis ? $1 : ''
  end 
  def vs
    /VS:?\s*(\d*)/ =~ self.popis ? $1 : ''
  end
  def ss
    /SS:?\s*(\d*)/ =~ self.popis ? $1 : ''
  end
  def reconciled?
    reconciled
  end
  def selected?
    self.selected == 1
  end
  def select
    self.selected = 1
  end
  def unselect
    self.selected = 0
  end
  def toggle_selected
    self.selected = (self.selected? ? 0 : 1)
  end
  def set_seq2(year, typ='pokladny')
    log = Logger.new(File.open(Rails.root.to_s+'/log/debug.log','a'))
    if seq2.blank?
      @seqs = []
      @seqs << account_from.next_seq2(year) if (account_from.pokladna? and typ =='pokladny') or (typ =='pcard') or (typ =='pcard2')
      @seqs << account_to.next_seq2(year) if (account_to.pokladna? and typ =='pokladny')
      self.seq2 = @seqs.join('/')
      return self.save!
    else
      fail('seq2 transferu ktery exportujete neni prazdne. Exportujete jiz vyexportovany transfer! '+self.inspect)
      return false
#       return self
    end
  end
  def set_default_seq
    @seqs = []
    @seqs << account_from.last_seq if account_from.envelope?
    @seqs << account_to.last_seq if account_to.envelope?
    self.seq = @seqs.join('/')
  end
  def lock_down_attributes_when_reconciled
    return unless reconciled?
    message = "se nesmí měnit když už je to reconcilováno"
    errors.add(:account_from_id, message) if account_from_id?
    errors.add(:account_to_id, message) if account_to_id_changed?
  end
  def nastav_date_inserted
    self.date_inserted = Time.now
    true
  end
  def existing_clones      # najde jestli transfer má nějaký klon v databázi
    Transfer.find_all_by_seq(self.seq)
  end
  def defaultize_and_save
    # v pripade ze je to envelope tak automaticky dopln chybejici seq
    if self.seq.blank? and self.account_from.andand.envelope?
      self.seq = self.account_from.last_seq
    end
    save
  end
  def defaultize_select_and_save
    select
    defaultize_and_save
  end
  def hospodarsky_rok
    datum.year
  end
  def star
    account_from.andand.most_specific? ? '' : '*'
  end
  alias_method :original_epoch, :epoch
  def epoch
    if self.default_epoch == true
      e = Epoch.get_default_for(self)
      self.epoch = e # cache
      e
    else
      self.original_epoch
    end
  end  
  def manual_epoch=(value)
    self.default_epoch = false
    self.epoch = value
  end
  def create_default_epoch_if_does_not_exist
    e = self.epoch
    if original_epoch.nil?
      e.epoch_id = e.id
    end
    if e.new_record?
      e.save
    else
      true # aby se pokracovalo v exkuci
    end    
  end
  def updated_at
    self.versions.last.andand.created_at
  end

  ## tags
  def tag_names_array
    self.tags.collect(&:nazev)
  end
  def tag_names_array=(value)
    self.tags = value.uniq.collect do |n|
      Tag.where(nazev: n.strip).first_or_create!
    end
  end  
  def tag_list
    self.tags.collect(&:nazev).join(", ")
  end
  def tag_list=(value)
    self.tag_names_array=value.split(",")
  end
  def tag_list_add=(value)
    self.tag_names_array = tag_names_array + value.split(",")
  end
 
end

